@@ -64,7 +64,7 @@ class behat_forms extends behat_base {
      * Fills a form with field/value data. More info in http://docs.moodle.org/dev/Acceptance_testing#Providing_values_to_steps.
      *
      * Backport of Moodle 2.7 method to help backporting
-     * features to 2.5.
+     * features to 2.6.
      *
      * @Given /^I set the following fields to these values:$/
      * @throws ElementNotFoundException Thrown by behat_base::find
@@ -130,18 +130,23 @@ class behat_forms extends behat_base {
      */
     protected function expand_all_fields() {
 
-        // behat_base::find() throws an exception if there are no elements, we should not fail a test because of this.
+        // We ensure that all the editors are loaded and we can interact with them.
+        $this->ensure_editors_are_loaded();
+
+        // We already know that we waited for the DOM and the JS to be loaded, even the editor
+        // so, we will use the reduced timeout as it is a common task and we should save time.
         try {
 
             // Expand fieldsets link.
-            $collapseexpandlink = $this->find('xpath', "//div[@class='collapsible-actions']" .
+            $xpath = "//div[@class='collapsible-actions']" .
                 "/descendant::a[contains(concat(' ', @class, ' '), ' collapseexpand ')]" .
-                "[not(contains(concat(' ', @class, ' '), ' collapse-all '))]"
-            );
+                "[not(contains(concat(' ', @class, ' '), ' collapse-all '))]";
+            $collapseexpandlink = $this->find('xpath', $xpath, false, false, self::REDUCED_TIMEOUT);
             $collapseexpandlink->click();
 
         } catch (ElementNotFoundException $e) {
-            // We continue if there are not expandable fields.
+            // The behat_base::find() method throws an exception if there are no elements,
+            // we should not fail a test because of this. We continue if there are not expandable fields.
         }
 
         // Different try & catch as we can have expanded fieldsets with advanced fields on them.
@@ -152,7 +157,9 @@ class behat_forms extends behat_base {
                 "[contains(concat(' ', normalize-space(@class), ' '), ' moreless-toggler')]";
 
             // We don't wait here as we already waited when getting the expand fieldsets links.
-            $showmores = $this->getSession()->getPage()->findAll('xpath', $showmorexpath);
+            if (!$showmores = $this->getSession()->getPage()->findAll('xpath', $showmorexpath)) {
+                return;
+            }
 
             // Funny thing about this, with findAll() we specify a pattern and each element matching the pattern is added to the array
             // with of xpaths with a [0], [1]... sufix, but when we click on an element it does not matches the specified xpath
@@ -173,7 +180,7 @@ class behat_forms extends behat_base {
      * Sets the specified value to the field.
      *
      * Backport of Moodle 2.7 method to help backporting
-     * features to 2.5.
+     * features to 2.6.
      *
      * @Given /^I set the field "(?P<field_string>(?:[^"]|\\")*)" to "(?P<field_value_string>(?:[^"]|\\")*)"$/
      * @throws ElementNotFoundException Thrown by behat_base::find
@@ -272,7 +279,7 @@ class behat_forms extends behat_base {
      * Checks, the field matches the value. More info in http://docs.moodle.org/dev/Acceptance_testing#Providing_values_to_steps.
      *
      * Backport of Moodle 2.7 method to help backporting
-     * features to 2.5.
+     * features to 2.6.
      *
      * @Then /^the field "(?P<field_string>(?:[^"]|\\")*)" matches value "(?P<field_value_string>(?:[^"]|\\")*)"$/
      * @throws ElementNotFoundException Thrown by behat_base::find
